package org.aohe.core.sane.utils;

import au.com.southsky.jfreesane.SaneDevice;
import au.com.southsky.jfreesane.SaneException;
import au.com.southsky.jfreesane.SaneSession;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.net.Ipv4Util;
import cn.hutool.core.net.NetUtil;
import cn.hutool.core.util.ByteUtil;
import cn.hutool.core.util.RuntimeUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import lombok.extern.slf4j.Slf4j;
import org.aohe.core.utils.CommandUtils;
import org.aohe.core.utils.SystemUtils;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;

@Slf4j
public class SaneSessionUtils {
    
    private static final Object lock = new Object();
    
    private static volatile SaneSession saneSession;
    
    private static volatile SaneDevice saneDevice;
    
    private static final RemoteConfig remoteConfig = RemoteConfig.getInstance();

    private static List<String> beUsedIp = new ArrayList<>(64);

    /**
     * 获取一个SaneSession实例，用于与SANE设备进行通信
     * @return SaneSession实例
     * @throws IOException 如果获取SaneSession实例时发生I/O错误
     */
    public static SaneSession getSaneSession() throws IOException {
        log.info("saneSession状态：{}", saneSession != null);
        if (saneSession == null) {
            synchronized(lock){
                if (saneSession == null) {
                    log.info("jFreeSaneConfig:{}", JSON.toJSONString(remoteConfig));
                    InetAddress address = InetAddress.getByName(remoteConfig.getIp());
                    saneSession = SaneSession.withRemoteSane(address);
                    log.info("初始化saneSession完成，状态：{}", saneSession != null);
                }
            }
        }
        return saneSession;
    }
    
    /**
     * 关闭SaneSession对象，如果对象不为null
     */
    public static void closeSaneSession() {
        if (saneSession != null) {
            try {
                saneSession.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            saneSession = null;
        }
    }
    
    /**
     * 获取所有的SANE驱动器设备列表
     *
     * @return SANE驱动器设备列表
     * @throws IOException    如果在通信中发生I/O错误
     * @throws SaneException  如果在与SANE交互中发生错误
     */
    public static List<SaneDevice> getSaneDrivers() throws IOException, SaneException {
        return getSaneSession().listDevices();
    }
    
    
    /**
     * 设置Sane设备
     * @param name 设备名称
     * @return 设备对象
     * @throws IOException 输入/输出异常
     * @throws SaneException Sane异常
     */
    public static SaneDevice setSaneDevice(String name) throws IOException, SaneException {
        if (saneDevice == null || !saneDevice.getName().equals(name)) {
            if (saneDevice != null && saneDevice.isOpen()) {
                // 关闭当前在使用的SANE设备
                saneDevice.close();
            }
            saneDevice = saneSession
                    .listDevices().stream().filter(s -> name.equals(s.getName())).findFirst().orElse(null);
        }
        return saneDevice;
    }
    
    /**
     * 获取Sane设备（如果不存在则返回null）
     * @return Sane设备
     */
    public static SaneDevice getDevice() {
        return saneDevice;
    }
    
    /**
     * 获取指定名称的Sane设备，如果设备不存在则返回null
     * @param name 设备名称
     * @return 指定名称的Sane设备
     * @throws RuntimeException 运行时异常
     */
    public static SaneDevice getDevice(String name) {
        try {
            return setSaneDevice(name);
        } catch (IOException | SaneException e) {
            throw new RuntimeException(e);
        }
    }
    
    
    /**
     * 获取一个正常的文件
     * 打开Sane设备
     * 通过Sane设备获取图像
     * 创建或更新最后访问时间的文件
     * 将图像写入文件
     * 返回文件
     * @throws RuntimeException - 如果发生I/O错误
     */
    public static File getSaneFile() throws IOException {
        try {
            saneDevice.open();
            BufferedImage bufferedImage =  saneDevice.acquireImage();
            File file = FileUtil.createTempFile("sane","png",true);
            ImageIO.write(bufferedImage, "png", file);
            return file;
        } catch (IOException | SaneException e) {
            throw new RuntimeException(e);
        }finally {
            if(saneDevice != null){
                saneDevice.close();
            }
        }
    }
    
    /**
     * 释放资源
     */
    public static void resource(){
        try {
            if(saneDevice != null){
                closeSaneDevice();
            }
            if(saneSession != null){
                closeSaneSession();
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * 关闭Sane设备
     */
    public static void closeSaneDevice() throws IOException {
        if(saneDevice != null){
            try {
                saneDevice.close();
            } catch (IOException e) {
                throw new IOException(e);
            }
            saneDevice = null;
        }
        
    }

    /**
     * 测试可用ip
     * @param ips ip
     */
    public static void beUsed(List<String> ips){
        try (ExecutorService executorService = SystemUtils.newFixedThreadPool(2);){
            for(String ip : ips){
                log.info("ip:{} , 开始检测", ip);
                executorService.submit(() -> SaneSessionUtils.beUsed(ip));
            }
        }catch (Exception e){
            log.info("查找出错 ");
        }
    }

    /**
     * 测试可用ip
     * @param ip ip
     */
    public static void beUsed(String ip){
        log.info("ip:{} , 可用性测试开始", ip);

        if(!NetUtil.ping(ip)){
            log.info("ip:{} , ping 测试不通过", ip);
            return;
        }
        try (SaneSession session = SaneSession.withRemoteSane(InetAddress.getByName(ip))){
            if(session == null){

                return;
            }
            log.info("ip:{} , session 打开成功", ip);
            //添加到可用列表
            beUsedIp.add(ip);
        } catch (IOException e) {
            log.info("ip:{} , session 打开失败", ip);
            throw new RuntimeException(e);
        }
    }

    public static void printUsedIp(){
        log.info("可用ip {} ", String.join("|", beUsedIp));
    }
}

